#include <stdarg.h>
#include "printf.h"
#include "types.h"
#include "lock.h"

//Spin_lock lock;
Ticket_lock lock;

ALWAYS_INLINE
void putc (char** const str, const char c)
{
    if (str) {
        **str = c;
        (*str)++;
    } else {
        char_out (c);
    }
}

void num (char** const str, uint64 n, unsigned base)
{
    char buffer[20];
    char* p = buffer + sizeof (buffer);

    uint32 h = static_cast<uint32>(n >> 32);
    uint32 l = static_cast<uint32>(n);
    uint32 m;

    do {
        asm volatile (
            "divl %[base]   ;"
            "xchg %%eax, %2 ;"
            "divl %[base]   ;"
            : "=a" (l),
              "=d" (m),
              "=r" (h)
            : "a" (h),
              "d" (0),
              "2" (l),
              [base] "rm" (base));

        m += m >= 10 ? 'a'-10 : '0';
        *--p = static_cast<char>(m); 

    } while (h|l);

    int c = buffer + sizeof (buffer) - p;

    while (c--)
        putc (str, *p++);
}

char const * _fmt (char** const str, char const *fmt, va_list& vl)
{
    bool alt = false;
    unsigned l = 0;

    for (;;) {
        switch (*++fmt) {
            case 0:
                return fmt;

            case '#':
                alt = true;
                continue;

            case 'l' :
                l++;
                continue;

            case '%':
                putc (str, '%');
                break;

            case 'c' :
                putc (str, static_cast<char> (va_arg (vl, int)));
                break;

            case 's' : {
                char* s = va_arg (vl, char*);
                while (*s)
                    putc (str, *s++);
                
                } break;

            case 'p' :
                putc (str, '0');
                putc (str, 'x');

                num (str, reinterpret_cast<mword>(va_arg (vl, void*)), 16);

                break;

            case 'd' : {
                int64 number;
                switch (l) {
                    case 0:  number = va_arg (vl, signed int);       break;
                    case 1:  number = va_arg (vl, signed long);      break;
                    default: number = va_arg (vl, signed long long); break;
                }
                if (number < 0) {
                    putc (str, '-');
                    number = -number;
                }

                num (str, number, 10);

                } break;

            case 'u' : {
            case 'x' :
                uint64 number;
                if (alt) {
                    putc (str, '0');
                    putc (str, 'x');
                }
                switch (l) {
                    case 0:  number = va_arg (vl, unsigned int);       break;
                    case 1:  number = va_arg (vl, unsigned long);      break;
                    default: number = va_arg (vl, unsigned long long); break;
                }

                num (str, number, *fmt == 'u' ? 10 : 16);
                     
                } break;

            default:
                break;
        }
        return fmt + 1;
    }

}

void print (char** const str, char const * fmt, va_list vl)
{
    for (;;) {
        if (*fmt == '%') {
            // format
            fmt = _fmt (str, fmt, vl);
        } else if (*fmt) {
            // default
            putc (str, *fmt++);
        } else {
            // '\0'
            if (str) putc (str, 0);
            return;
        }
    }
}

void printf (char const * fmt, ...)
{
    lock.lock();
    va_list vl;
    va_start (vl, fmt);
    print (0, fmt, vl);
    va_end (vl);
    lock.unlock();
}

void sprintf(char *str, const char *format, ...)
{
    va_list args;
    va_start (args, format);
    print (&str, format, args);
    va_end (args);
}

// thanks to gcc's printf optimization : undefined reference to `puts'
extern "C"
int puts(const char *s)
{
    printf("%s\n",s);
}

extern "C"
int putchar (int i)
{
    char c = i;
    putc (0,c);
}
